"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.graphqlMiddleware = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const chokidar_1 = __importDefault(require("chokidar"));
const consola_1 = __importDefault(require("consola"));
const serverMiddleware_1 = __importDefault(require("./../serverMiddleware"));
const graphqlImport_1 = __importDefault(require("./graphqlImport"));
const logger = consola_1.default.withTag('nuxt-graphql-middleware');
const PLUGIN_PATH = path_1.default.resolve(__dirname, './../plugin/index.js');
var FileType;
(function (FileType) {
    FileType["Query"] = "query";
    FileType["Mutation"] = "mutation";
})(FileType || (FileType = {}));
function resolveGraphqlFile(file, resolver) {
    return fs_1.default.promises
        .readFile(file)
        .then((buffer) => buffer.toString())
        .then((source) => graphqlImport_1.default(source, resolver));
}
function writeSource(dest, filePath, source) {
    const fileName = path_1.default.basename(filePath);
    const out = path_1.default.resolve(dest, fileName);
    return fs_1.default.promises.writeFile(out, source);
}
function resolveGraphql(files, map, resolver, filesMap, type, outputPath) {
    return Promise.all(Object.keys(files).map((name) => {
        const filePath = files[name];
        const file = resolver(filePath);
        return resolveGraphqlFile(file, resolver).then((source) => {
            map.set(name, source);
            if (outputPath) {
                writeSource(outputPath, filePath, source);
            }
            filesMap.set(file, {
                type,
                name,
                file,
            });
        });
    }));
}
const graphqlMiddleware = function () {
    var _a, _b, _c, _d, _e, _f;
    return __awaiter(this, void 0, void 0, function* () {
        const options = this.options;
        const provided = (this.options.graphqlMiddleware ||
            {});
        const config = {
            graphqlServer: provided.graphqlServer || '',
            endpointNamespace: provided.endpointNamespace || '/__graphql_middleware',
            debug: provided.debug || options.dev,
            queries: provided.queries || {},
            mutations: provided.mutations || {},
            outputPath: provided.outputPath || '',
            server: provided.server,
            plugin: {
                enabled: !!((_a = provided.plugin) === null || _a === void 0 ? void 0 : _a.enabled),
                cacheInBrowser: !!((_b = provided.plugin) === null || _b === void 0 ? void 0 : _b.cacheInBrowser),
                cacheInServer: !!((_c = provided.plugin) === null || _c === void 0 ? void 0 : _c.cacheInServer),
            },
        };
        const resolver = this.nuxt.resolver.resolvePath;
        if ((_d = config.plugin) === null || _d === void 0 ? void 0 : _d.enabled) {
            this.addPlugin({
                src: PLUGIN_PATH,
                options: {
                    namespace: config.endpointNamespace,
                    cacheInBrowser: ((_e = config.plugin) === null || _e === void 0 ? void 0 : _e.cacheInBrowser) ? 'true' : 'false',
                    cacheInServer: ((_f = config.plugin) === null || _f === void 0 ? void 0 : _f.cacheInServer) ? 'true' : 'false',
                },
            });
        }
        const fileMap = new Map();
        const queries = new Map();
        const mutations = new Map();
        const outputPath = config.outputPath ? resolver(config.outputPath) : '';
        if (outputPath) {
            yield mkdirp_1.default(outputPath);
        }
        function build() {
            logger.log('Building GraphQL files...');
            return Promise.all([
                resolveGraphql(config.queries, queries, resolver, fileMap, FileType.Query, outputPath),
                resolveGraphql(config.mutations, mutations, resolver, fileMap, FileType.Mutation, outputPath),
            ]).then(() => {
                logger.success('Finished building GraphQL files');
            });
        }
        function watchFiles() {
            const ignored = ['node_modules', '.nuxt'];
            if (config.outputPath) {
                ignored.push(config.outputPath);
            }
            const filesWatcher = (options._filesWatcher = chokidar_1.default.watch('./**/*.graphql', {
                ignoreInitial: true,
                ignored,
            }));
            if (filesWatcher) {
                logger.info('Watching for query changes');
                filesWatcher.on('change', () => {
                    build();
                });
            }
        }
        build().then(() => {
            if (options.debug) {
                logger.info('Available queries and mutations:');
                console.table(Array.from(fileMap.entries()).map(([_key, value]) => value));
            }
        });
        if (this.nuxt.options.dev) {
            watchFiles();
        }
        this.addServerMiddleware({
            path: config.endpointNamespace,
            handler: serverMiddleware_1.default(config.graphqlServer, queries, mutations, config.server),
        });
    });
};
exports.graphqlMiddleware = graphqlMiddleware;
